# Copyright © 2020 Gregor Reitzenstein
# Licensed under the MIT License:
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
# DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
# OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


@0x9e1c146a27dcc635;

using Rust = import "rust.capnp";
$Rust.parentModule("schema");

using CSharp = import "csharp.capnp";
$CSharp.namespace("FabAccessAPI.Schema");

using Auth = import "auth.capnp";
using Api = import "api.capnp";

struct Message {
    union {
        greet @0 :Greeting;
        # Be polite and say hello to the other end

        leave @1 :Leave;

        auth @2 :Auth.AuthMessage;
        # Start an authenticaion exchange

        # TODO: RPC bootstrapping
        api @3 :Api.FabAccess;
    }
}

struct Greeting {
    # Be nice and say hello to each other first
    # A client sends this message to a server first thing, a server then replies
    # similarly

    # Hello, I'm …
    host @0 :Text; # non-qualified Hostname of the client/server, as applicable

    # using the program …
    program @1 :Text; # SHOULD be of the form `programname-version`

    # and speak the API version …
    major @2 :UInt32; # The major part of the API version
    minor @3 :UInt32; # The minor part of the API version
}

struct Leave {
    # Be nice and tell the other side before aborting a connection
    enum Reason {
        other @0;
        # None of the more specific reasons.  An implementation SHOULD provide
        # more (human-readable) information in the `message` field.

        incompatible @1;
        # The API versions are in some way incompatible.  Doesn't need much of
        # an explanation so an implementation MAY leave the message field
        # empty.
    }

    reason @0 :Reason;
    message @1 :Text;   
    # An implementation SHOULD send a human-readable message along, with the
    # exception of reasons that are self-explanatory (e.g. incompatible versions)
}
